useDynLib(randomForest, .registration=TRUE)

export(combine, getTree, grow, importance, margin, MDSplot, na.roughfix,
       partialPlot, randomForest, rfImpute, treesize, tuneRF,
       varImpPlot, varUsed, rfNews, outlier, classCenter, rfcv)

S3method(print, randomForest)
S3method(predict, randomForest)
S3method(plot, randomForest)
S3method(plot, margin)
S3method(margin, randomForest)
S3method(partialPlot, randomForest)
S3method(grow, randomForest)
S3method(importance, randomForest)
S3method(outlier, randomForest)
S3method(outlier, default)

S3method(grow, default)
S3method(importance, default)
S3method(margin, default)
S3method(partialPlot, default)

S3method(randomForest, formula)
S3method(randomForest, default)
S3method(rfImpute, formula)
S3method(rfImpute, default)

S3method(na.roughfix, data.frame)
S3method(na.roughfix, default)

importFrom("grDevices", "rainbow")
importFrom("graphics", "axis", "barplot", "dotchart", "lines",
           "matplot", "mtext", "pairs", "par", "plot", "plot.default",
           "points")
importFrom("stats", "delete.response", "mad", "median", "model.frame",
           "model.response", "na.fail", "na.omit", "napredict",
           "predict", "quantile", "reformulate", "terms", "update",
           "var", "weighted.mean")
